@@ -78,6 +78,7 @@ class DefaultDeviceHandler(object):
         """
         return [
             "urn:ietf:params:netconf:base:1.0",
+            "urn:ietf:params:netconf:base:1.1",
             "urn:ietf:params:netconf:capability:writable-running:1.0",
             "urn:ietf:params:netconf:capability:candidate:1.0",
             "urn:ietf:params:netconf:capability:confirmed-commit:1.0",
@@ -160,7 +161,10 @@ class DefaultDeviceHandler(object):
         Return True/False depending on found match.
 
         """
-        error_text = error_text.lower().strip()
+        if error_text is not None:
+            error_text = error_text.lower().strip()
+        else:
+            error_text = 'no error given'
 
         # Compare the error text against all the exempt errors.
         for ex in self._exempt_errors_exact_match:
