@@ -1,4 +1,5 @@
 # Copyright 2009 Shikhar Bhushan
+# Copyright 2014 Leonidas Poulopoulos
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,11 +22,10 @@ from threading import Thread, Lock, Event
 from ncclient.xml_ import *
 from ncclient.capabilities import Capabilities
 
-from errors import TransportError
+from errors import TransportError, SessionError
 
 import logging
 logger = logging.getLogger('ncclient.transport.session')
-logger.setLevel(logging.WARNING)
 
 
 class Session(Thread):
@@ -90,8 +90,10 @@ class Session(Thread):
         self.send(HelloHandler.build(self._client_capabilities, self._device_handler))
         logger.debug('starting main loop')
         self.start()
-        # we expect server's hello message
-        init_event.wait()
+        # we expect server's hello message, if server doesn't responds in 60 seconds raise exception
+        init_event.wait(60)
+        if not init_event.is_set():
+            raise SessionError("Capability exchange timed out")
         # received hello message or an error happened
         self.remove_listener(listener)
         if error[0]:
@@ -221,7 +223,9 @@ class HelloHandler(SessionListener):
     def build(capabilities, device_handler):
         "Given a list of capability URI's returns <hello> message XML string"
         if device_handler:
-            xml_namespace_kwargs = device_handler.get_xml_base_namespace_dict()
+            # This is used as kwargs dictionary for lxml's Element() function.
+            # Therefore the arg-name ("nsmap") is used as key here.
+            xml_namespace_kwargs = { "nsmap" : device_handler.get_xml_base_namespace_dict() }
         else:
             xml_namespace_kwargs = {}
         hello = new_ele("hello", **xml_namespace_kwargs)
