@@ -1,3 +1,4 @@
+# Copyright 2012 Vaibhav Bajpai
 # Copyright 2009 Shikhar Bhushan
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,7 @@
 # limitations under the License.
 
 import os
+import sys
 import socket
 import getpass
 from binascii import hexlify
@@ -25,14 +27,17 @@ import paramiko
 
 from errors import AuthenticationError, SessionCloseError, SSHError, SSHUnknownHostError
 from session import Session
+from ncclient.xml_ import *
 
 import logging
 logger = logging.getLogger("ncclient.transport.ssh")
-logger.setLevel(logging.WARNING)
-logging.getLogger("paramiko").setLevel(logging.DEBUG)
 
 BUF_SIZE = 4096
+# v1.0: RFC 4742
 MSG_DELIM = "]]>]]>"
+# v1.1: RFC 6242
+END_DELIM = '\n##\n'
+
 TICK = 0.1
 
 def default_unknown_host_cb(host, fingerprint):
@@ -69,17 +74,34 @@ class SSHSession(Session):
         self._channel_name = None
         self._buffer = StringIO() # for incoming data
         # parsing-related, see _parse()
-        self._parsing_state = 0
-        self._parsing_pos = 0
         self._device_handler = device_handler
+        self._parsing_state10 = 0
+        self._parsing_pos10 = 0
+        self._parsing_pos11 = 0
+        self._parsing_state11 = 0
+        self._expchunksize = 0
+        self._curchunksize = 0
+        self._inendpos = 0
+        self._size_num_list = []
+        self._message_list = []
 
     def _parse(self):
         "Messages ae delimited by MSG_DELIM. The buffer could have grown by a maximum of BUF_SIZE bytes everytime this method is called. Retains state across method calls and if a byte has been read it will not be considered again."
+        return self._parse10()
+
+    def _parse10(self):
+
+        """Messages are delimited by MSG_DELIM. The buffer could have grown by
+        a maximum of BUF_SIZE bytes everytime this method is called. Retains
+        state across method calls and if a byte has been read it will not be
+        considered again."""
+
+        logger.debug("parsing netconf v1.0")
         delim = MSG_DELIM
         n = len(delim) - 1
-        expect = self._parsing_state
+        expect = self._parsing_state10
         buf = self._buffer
-        buf.seek(self._parsing_pos)
+        buf.seek(self._parsing_pos10)
         while True:
             x = buf.read(1)
             if not x: # done reading
@@ -111,14 +133,155 @@ class SSHSession(Session):
                 buf.seek(0)
                 expect = 0
         self._buffer = buf
-        self._parsing_state = expect
-        self._parsing_pos = self._buffer.tell()
+        self._parsing_state10 = expect
+        self._parsing_pos10 = self._buffer.tell()
+
+    def _parse11(self):
+        logger.debug("parsing netconf v1.1")
+        expchunksize = self._expchunksize
+        curchunksize = self._curchunksize
+        idle, instart, inmsg, inbetween, inend = range(5)
+        state = self._parsing_state11
+        inendpos = self._inendpos
+        num_list = self._size_num_list
+        MAX_STARTCHUNK_SIZE = 10 # 4294967295
+        pre = 'invalid base:1:1 frame'
+        buf = self._buffer
+        buf.seek(self._parsing_pos11)
+        message_list = self._message_list # a message is a list of chunks
+        chunk_list = []   # a chunk is a list of characters
+
+        while True:
+            x = buf.read(1)
+            if not x:
+                logger.debug('No more data to read')
+                # Store the current chunk to the message list
+                chunk = ''.join(chunk_list)
+                message_list.append(chunk)
+                break # done reading
+            logger.debug('x: %s', x)
+            if state == idle:
+                if x == '\n':
+                    state = instart
+                    inendpos = 1
+                else:
+                    logger.debug('%s (%s: expect newline)'%(pre, state))
+                    raise Exception
+            elif state == instart:
+                if inendpos == 1:
+                    if x == '#':
+                        inendpos += 1
+                    else:
+                        logger.debug('%s (%s: expect "#")'%(pre, state))
+                        raise Exception
+                elif inendpos == 2:
+                    if x.isdigit():
+                        inendpos += 1 # == 3 now #
+                        num_list.append(x)
+                    else:
+                        logger.debug('%s (%s: expect digit)'%(pre, state))
+                        raise Exception
+                else:
+                    if inendpos == MAX_STARTCHUNK_SIZE:
+                        logger.debug('%s (%s: no. too long)'%(pre, state))
+                        raise Exception
+                    elif x == '\n':
+                        num = ''.join(num_list)
+                        num_list = [] # Reset num_list
+                        try: num = long(num)
+                        except:
+                            logger.debug('%s (%s: invalid no.)'%(pre, state))
+                            raise Exception
+                        else:
+                            state = inmsg
+                            expchunksize = num
+                            logger.debug('response length: %d'%expchunksize)
+                            curchunksize = 0
+                            inendpos += 1
+                    elif x.isdigit():
+                        inendpos += 1 # > 3 now #
+                        num_list.append(x)
+                    else:
+                        log.debug('%s (%s: expect digit)'%(pre, state))
+                        raise Exception
+            elif state == inmsg:
+                chunk_list.append(x)
+                curchunksize += 1
+                chunkleft = expchunksize - curchunksize
+                if chunkleft == 0:
+                    inendpos = 0
+                    state = inbetween
+                    chunk = ''.join(chunk_list)
+                    message_list.append(chunk)
+                    chunk_list = [] # Reset chunk_list    
+                    logger.debug('parsed new chunk: %s'%(chunk))
+            elif state == inbetween:
+                if inendpos == 0:
+                    if x == '\n': inendpos += 1
+                    else:
+                        logger.debug('%s (%s: expect newline)'%(pre, state))
+                        raise Exception
+                elif inendpos == 1:
+                    if x == '#': inendpos += 1
+                    else:
+                        logger.debug('%s (%s: expect "#")'%(pre, state))
+                        raise Exception
+                else:
+                    inendpos += 1 # == 3 now #
+                    if x == '#':
+                        state = inend
+                    elif x.isdigit():
+                        # More trunks
+                        state = instart
+                        num_list = []
+                        num_list.append(x)
+                    else:
+                        logger.debug('%s (%s: expect "#")'%(pre, state))
+                        raise Exception
+            elif state == inend:
+                if inendpos == 3:
+                    if x == '\n':
+                        inendpos = 0
+                        state = idle
+                        logger.debug('dispatching message')
+                        self._dispatch_message(''.join(message_list))
+                        # reset
+                        rest = buf.read()
+                        buf = StringIO()
+                        buf.write(rest)
+                        buf.seek(0)
+                        message_list = []
+                        self._message_list = message_list
+                        chunk_list = []
+                        expchunksize = chunksize = 0
+                        parsing_state11 = idle
+                        inendpos = parsing_pos11 = 0
+                        break
+                    else:
+                        logger.debug('%s (%s: expect newline)'%(pre, state))
+                        raise Exception
+            else:
+                logger.debug('%s (%s invalid state)'%(pre, state))
+                raise Exception
+
+        self._expchunksize = expchunksize
+        self._curchunksize = curchunksize
+        self._parsing_state11 = state
+        self._inendpos = inendpos
+        self._size_num_list = num_list
+        self._buffer = buf
+        self._parsing_pos11 = self._buffer.tell()
+        logger.debug('parse11 ending ...')
+
 
     def load_known_hosts(self, filename=None):
-        """Load host keys from an openssh :file:`known_hosts`-style file. Can be called multiple times.
+
+        """Load host keys from an openssh :file:`known_hosts`-style file. Can
+        be called multiple times.
 
         If *filename* is not specified, looks in the default locations i.e. :file:`~/.ssh/known_hosts` and :file:`~/ssh/known_hosts` for Windows.
         """
+
         if filename is None:
             filename = os.path.expanduser('~/.ssh/known_hosts')
             try:
@@ -136,7 +299,9 @@ class SSHSession(Session):
     def close(self):
         if self._transport.is_active():
             self._transport.close()
+        self._channel = None
         self._connected = False
+        
 
     # REMEMBER to update transport.rst if sig. changes, since it is hardcoded there
     def connect(self, host, port=830, timeout=None, unknown_host_cb=default_unknown_host_cb,
@@ -167,10 +332,12 @@ class SSHSession(Session):
 
         *look_for_keys* enables looking in the usual locations for ssh keys (e.g. :file:`~/.ssh/id_*`)
 
-        *ssh_config* enables parsing of an OpenSSH configuration file, if set to its path, e.g. ~/.ssh/config
+        *ssh_config* enables parsing of an OpenSSH configuration file, if set to its path, e.g. :file:`~/.ssh/config` or to True (in this case, use :file:`~/.ssh/config`).
         """
         # Optionaly, parse .ssh/config
         config = {}
+        if ssh_config is True:
+            ssh_config = "~/.ssh/config" if sys.platform != "win32" else "~/ssh/config"
         if ssh_config is not None:
             config = paramiko.SSHConfig()
             config.parse(open(os.path.expanduser(ssh_config)))
@@ -214,11 +381,11 @@ class SSHSession(Session):
 
         # host key verification
         server_key = t.get_remote_server_key()
-        known_host = self._host_keys.check(host, server_key)
 
         fingerprint = _colonify(hexlify(server_key.get_fingerprint()))
 
         if hostkey_verify:
+            known_host = self._host_keys.check(host, server_key)
             if not known_host and not unknown_host_cb(host, fingerprint):
                 raise SSHUnknownHostError(host, fingerprint)
 
@@ -326,6 +493,9 @@ class SSHSession(Session):
     def run(self):
         chan = self._channel
         q = self._q
+
+        def start_delim(data_len): return '\n#%s\n'%(data_len)
+
         try:
             while True:
                 # select on a paramiko ssh channel object does not ever return it in the writable list, so channels don't exactly emulate the socket api
@@ -335,17 +505,51 @@ class SSHSession(Session):
                     data = chan.recv(BUF_SIZE)
                     if data:
                         self._buffer.write(data)
-                        self._parse()
+                        if self._server_capabilities:
+                            if 'urn:ietf:params:netconf:base:1.1' in self._server_capabilities and 'urn:ietf:params:netconf:base:1.1' in self._client_capabilities:
+                                logger.debug("Selecting netconf:base:1.1 for encoding")
+                                self._parse11()
+                            elif 'urn:ietf:params:netconf:base:1.0' in self._server_capabilities or 'urn:ietf:params:xml:ns:netconf:base:1.0' in self._server_capabilities or 'urn:ietf:params:netconf:base:1.0' in self._client_capabilities:
+                                logger.debug("Selecting netconf:base:1.0 for encoding")
+                                self._parse10()
+                            else: raise Exception
+                        else:
+                            self._parse10() # HELLO msg uses EOM markers.
                     else:
                         raise SessionCloseError(self._buffer.getvalue())
                 if not q.empty() and chan.send_ready():
                     logger.debug("Sending message")
-                    data = q.get() + MSG_DELIM
-                    while data:
-                        n = chan.send(data)
-                        if n <= 0:
-                            raise SessionCloseError(self._buffer.getvalue(), data)
-                        data = data[n:]
+                    data = q.get()
+                    try:
+                        # send a HELLO msg using v1.0 EOM markers.
+                        validated_element(data, tags='{urn:ietf:params:xml:ns:netconf:base:1.0}hello')
+                        data = "%s%s"%(data, MSG_DELIM)
+                    except XMLError:
+                        # this is not a HELLO msg
+                        # we publish v1.1 support
+                        if 'urn:ietf:params:netconf:base:1.1' in self._client_capabilities:
+                            if self._server_capabilities:
+                                if 'urn:ietf:params:netconf:base:1.1' in self._server_capabilities:
+                                    # send using v1.1 chunked framing
+                                    data = "%s%s%s"%(start_delim(len(data)), data, END_DELIM)
+                                elif 'urn:ietf:params:netconf:base:1.0' in self._server_capabilities or 'urn:ietf:params:xml:ns:netconf:base:1.0' in self._server_capabilities:
+                                    # send using v1.0 EOM markers
+                                    data = "%s%s"%(data, MSG_DELIM)
+                                else: raise Exception
+                            else:
+                                logger.debug('HELLO msg was sent, but server capabilities are still not known')
+                                raise Exception
+                        # we publish only v1.0 support
+                        else:
+                            # send using v1.0 EOM markers
+                            data = "%s%s"%(data, MSG_DELIM)
+                    finally:
+                        logger.debug("Sending: %s", data)
+                        while data:
+                            n = chan.send(data)
+                            if n <= 0:
+                                raise SessionCloseError(self._buffer.getvalue(), data)
+                            data = data[n:]
         except Exception as e:
             logger.debug("Broke out of main loop, error=%r", e)
             self._dispatch_error(e)
